==> bayonne-0.4.3.tar.gz: bayonne-0.4.3/drivers/vpb/driver.cpp <==
// Copyright (C) 2000 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// As a special exception to the GNU General Public License, permission is
// granted for additional uses of the text contained in its release
// of Bayonne as noted here.
//
// This exception is that permission is hereby granted to link Bayonne 
// with  the Pika MonteCarlo libraries to produce a executable image
// without requiring MonteCarlo itself to be supplied in source form so
// long as each source file so linked contains this exclusion.
//
// This exception does not however invalidate any other reasons why
// the resulting executable file might be covered by the GNU General
// public license or invalidate the licensing requirements of any
// other component or library.
//
// This exception applies only to the code released by OST under the
// name Bayonne.  If you copy code from other releases into a copy of
// Bayonne, as the General Public License permits, the exception does not
// apply to the code that you add in this way.  To avoid misleading
// anyone as to the status of such modified files, you must delete
// this exception notice from them.
//
// If you write modifications of your own to Bayonne, it is your choice
// whether to permit this exception to apply to your modifications.
// If you do not wish that, delete this exception notice, at which
// point the terms of your modification would be covered under the GPL
// as explicitly stated in "COPYING".

#include "driver.h"
#include "ioports.h"

VPBConfig::VPBConfig() :
Keydata("/bayonne/vpb")
{
	static KEYDEF defkeys[] = {
	{"first", "300"},
	{"cards", "1"},
	{"channels", "4"},
	{"messagesize", "128"},
	{"transmitsize", "1000"},
	{"receivesize", "1000"},
	{"framesize", "160"},
	{"relaysize", "16000"},
	{"firmware", "/etc/vpbmain.out"},
	{NULL, NULL}};

	Load(defkeys);
}

VPBDriver::VPBDriver() :
Driver()
{
	threads = NULL;
	ports = NULL;
	groups = NULL;
	port_count = 0;

	if(getuid())
	{
		slog(SLOG_CRITICAL) << "vpb driver requires root" << endl;
		throw((Driver *)this);
	}
	slog(SLOG_INFO) << "vpb driver loaded; capacity=" << getCards() << " cards" << endl;
}

VPBDriver::~VPBDriver()
{
	Stop();
	if(ports)
		delete ports;

	if(groups)
		delete groups;
}

int VPBDriver::Start(void)
{
	int count = 0;
	int cards = getCards();

	if(active)
	{
		slog(SLOG_ERROR) << "vpb: driver already started" << endl;
		return 0;
	}

	if(iopriv(1))
	{
		slog(SLOG_CRITICAL) << "vpb: cannot access io ports" << endl;
		return 0;
	}

	threads = new VPBThread[cards];

	slog(SLOG_INFO) << "vpb: initalized " << VPBThread::cards << " of " << VPBThread::count << " card(s) successfully..." << endl;
	active = true;
	return count;
}

void VPBDriver::Stop(void)
{
	if(!active)
		return;

	if(threads)
	{
		slog(SLOG_INFO) << "vpb: threads exiting..." << endl;
		delete threads;
		threads = NULL;
	}

	iopriv(0);

	if(ports)
	{
		delete ports;
		ports = NULL;
	}

	active = false;
	slog(SLOG_INFO) << "vpb: stopping..." << endl;
}

Trunk *VPBDriver::getTrunkPort(int id)
{
	if(id < 0 || id >= port_count)
		return NULL;

	if(!ports)
		return NULL;

	return (Trunk *)ports[id];
}

VPBDriver vpbivr;

==> bayonne-0.4.4.tar.gz: bayonne-0.4.4/drivers/vpb/driver.cpp <==
// Copyright (C) 2000 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "driver.h"

VPBConfig::VPBConfig() :
Keydata("/bayonne/vpb")
{
	static KEYDEF defkeys[] = {
	{"cards", "1"},
	{"first", "300"},
	{"hooktime", "300"},
	{"firmware", "/etc/vpbmain.out"},
	{NULL, NULL}};

	Load(defkeys);
}

VPBDriver::VPBDriver() :
Driver()
{
	threads = NULL;
	port_count = getCards() * 4;
	status = VPBTrunk::status;

	ports = new VPBTrunk *[port_count];
	groups = new TrunkGroup *[port_count];
	memset(ports, 0, sizeof(VPBTrunk *) * port_count);
	memset(groups, 0, sizeof(TrunkGroup *) * port_count);

	memset(status, ' ', sizeof(VPBTrunk::status));
	if(getuid())
	{
		slog(SLOG_CRITICAL) << "vpb driver requires root" << endl;
		throw((Driver *)this);
	}
	slog(SLOG_INFO) << "vpb driver loaded; capacity=" << getCards() << " cards" << endl;
}

VPBDriver::~VPBDriver()
{
	Stop();
	if(ports)
		delete ports;

	if(groups)
		delete groups;
}

int VPBDriver::Start(void)
{
	int count, ts;
	int cards = getCards();
	int offsets[16];
	int first = getOffset();

	// assume from port 0x3x0 upward

	for(count = 0; count < cards; ++count)
		offsets[count] = first + 16 * count; 

	if(active)
	{
		slog(SLOG_ERROR) << "vpb: driver already started" << endl;
		return 0;
	}

	vpb_start(cards, offsets, (char *)getFirmware());

	threads = new VPBThread[cards];

	for(count = 0; count < cards; ++count)
		for(ts = 0; ts < 4; ++ts)
			ports[count * 4 + ts] = new VPBTrunk(&threads[count], ts);

	for(count = 0; count < cards; ++count)
		threads[count].Start();

	slog(SLOG_INFO) << "vpb: initalized " << cards * 4 << " ports successfully..." << endl;
	active = true;
	return cards * 4;
}

void VPBDriver::Stop(void)
{
	unsigned port;

	if(!active)
		return;

	for(port = 0; port < port_count; ++port)
		if(ports[port])
			delete ports[port];

	if(threads)
	{
		slog(SLOG_INFO) << "vpb: threads exiting..." << endl;
		delete[] threads;
		threads = NULL;
	}

	if(ports)
	{
		delete ports;
		ports = NULL;
	}
	vpb_stop();
	active = false;
	slog(SLOG_INFO) << "vpb: stopping..." << endl;
}

Trunk *VPBDriver::getTrunkPort(int id)
{
	if(id < 0 || id >= port_count)
		return NULL;

	if(!ports)
		return NULL;

	return (Trunk *)ports[id];
}

VPBDriver vpbivr;
